/*
 * Copyright © 2022-2024 Harsh Shandilya.
 * Use of this source code is governed by an MIT-style
 * license that can be found in the LICENSE file or at
 * https://opensource.org/licenses/MIT.
 */
package dev.msfjarvis.claw.common.comments

import androidx.compose.runtime.Composable
import dev.msfjarvis.claw.common.bonsai.node.Branch
import dev.msfjarvis.claw.common.bonsai.node.Leaf
import dev.msfjarvis.claw.common.bonsai.tree.Tree
import dev.msfjarvis.claw.common.bonsai.tree.TreeScope
import dev.msfjarvis.claw.common.bonsai.tree.tree
import dev.msfjarvis.claw.database.local.PostComments
import dev.msfjarvis.claw.model.Comment
import kotlinx.collections.immutable.ImmutableList
import kotlinx.collections.immutable.toPersistentList

internal data class CommentNode(
  val comment: Comment,
  val isUnread: Boolean = false,
  var isExpanded: Boolean = true,
)

internal fun createCommentList(
  comments: List<Comment>,
  commentState: PostComments?,
): ImmutableList<CommentNode> {
  val isUnread = { id: String -> commentState?.commentIds?.contains(id) == false }
  return comments
    .map { comment -> CommentNode(comment, isUnread(comment.shortId), isExpanded = true) }
    .toPersistentList()
}

@Composable
internal fun createTree(
  comments: ImmutableList<CommentNode>,
  htmlConverter: HTMLConverter,
  openUserProfile: (String) -> Unit,
): Tree<CommentNode> {
  val commentsMap = comments.groupBy { it.comment.parentComment }

  @Composable
  fun TreeScope.buildTree(commentId: String?) {
    val comment = comments.find { it.comment.shortId == commentId }!!
    val children = commentsMap[commentId]
    return if (children.isNullOrEmpty()) {
      Leaf(
        content = comment,
        customIcon = {},
        customName = { CommentEntry(comment, htmlConverter, openUserProfile) },
      )
    } else {
      Branch(
        content = comment,
        children = { children.map { buildTree(it.comment.shortId) } },
        customIcon = {},
        customName = { CommentEntry(comment, htmlConverter, openUserProfile) },
      )
    }
  }

  return tree {
    val rootCommentIds =
      comments.filter { it.comment.parentComment == null }.map { it.comment.shortId }
    rootCommentIds.map { buildTree(it) }
  }
}
