/*
 * Copyright © 2021-2024 Harsh Shandilya.
 * Use of this source code is governed by an MIT-style
 * license that can be found in the LICENSE file or at
 * https://opensource.org/licenses/MIT.
 */
package dev.msfjarvis.claw.common.comments

import androidx.compose.foundation.layout.Box
import androidx.compose.foundation.layout.PaddingValues
import androidx.compose.foundation.layout.fillMaxSize
import androidx.compose.foundation.layout.padding
import androidx.compose.material3.MaterialTheme
import androidx.compose.material3.Surface
import androidx.compose.material3.Text
import androidx.compose.runtime.Composable
import androidx.compose.runtime.LaunchedEffect
import androidx.compose.runtime.getValue
import androidx.compose.runtime.produceState
import androidx.compose.ui.Alignment
import androidx.compose.ui.Modifier
import androidx.compose.ui.graphics.RectangleShape
import androidx.compose.ui.unit.dp
import com.github.michaelbull.result.coroutines.runSuspendCatching
import com.github.michaelbull.result.fold
import dev.msfjarvis.claw.common.NetworkState
import dev.msfjarvis.claw.common.NetworkState.Error
import dev.msfjarvis.claw.common.NetworkState.Loading
import dev.msfjarvis.claw.common.NetworkState.Success
import dev.msfjarvis.claw.common.bonsai.Bonsai
import dev.msfjarvis.claw.common.bonsai.BonsaiStyle
import dev.msfjarvis.claw.common.posts.PostActions
import dev.msfjarvis.claw.common.ui.NetworkError
import dev.msfjarvis.claw.common.ui.ProgressBar
import dev.msfjarvis.claw.database.local.PostComments
import dev.msfjarvis.claw.model.Comment
import dev.msfjarvis.claw.model.UIPost

@Composable
private fun CommentsPageInternal(
  details: UIPost,
  postActions: PostActions,
  htmlConverter: HTMLConverter,
  commentState: PostComments?,
  markSeenComments: (String, List<Comment>) -> Unit,
  openUserProfile: (String) -> Unit,
  modifier: Modifier = Modifier,
) {
  val tree =
    createTree(createCommentList(details.comments, commentState), htmlConverter, openUserProfile)
  LaunchedEffect(Unit) {
    // Start off the tree in expanded state, then let Bonsai take care of it.
    tree.expandAll()

    markSeenComments(details.shortId, details.comments)
  }

  Surface(color = MaterialTheme.colorScheme.surfaceVariant, modifier = modifier) {
    Bonsai(
      tree = tree,
      style =
        BonsaiStyle(
          nodePadding = PaddingValues(0.dp),
          useHorizontalScroll = false,
          nodeShape = RectangleShape,
        ),
    ) {
      CommentsHeader(
        post = details,
        postActions = postActions,
        htmlConverter = htmlConverter,
        openUserProfile = openUserProfile,
      )
      Text(
        text = "Comments",
        style = MaterialTheme.typography.labelLarge,
        modifier = Modifier.padding(horizontal = 16.dp, vertical = 12.dp),
      )
    }
  }
}

@Suppress("UNCHECKED_CAST")
@Composable
fun CommentsPage(
  postId: String,
  postActions: PostActions,
  htmlConverter: HTMLConverter,
  getSeenComments: suspend (String) -> PostComments?,
  markSeenComments: (String, List<Comment>) -> Unit,
  modifier: Modifier = Modifier,
  openUserProfile: (String) -> Unit,
) {
  val postDetails by
    produceState<NetworkState>(Loading) {
      runSuspendCatching { postActions.getComments(postId) }
        .fold(
          success = { details -> value = Success(details) },
          failure = { value = Error(error = it, description = "Failed to load comments") },
        )
    }
  val commentState by
    produceState<PostComments?>(initialValue = null) { value = getSeenComments(postId) }

  when (postDetails) {
    is Success<*> -> {
      CommentsPageInternal(
        details = (postDetails as Success<UIPost>).data,
        postActions = postActions,
        htmlConverter = htmlConverter,
        commentState = commentState,
        markSeenComments = markSeenComments,
        openUserProfile = openUserProfile,
        modifier = modifier.fillMaxSize(),
      )
    }
    is Error -> {
      val error = postDetails as Error
      Box(modifier = Modifier.fillMaxSize()) {
        NetworkError(
          label = error.description,
          error = error.error,
          modifier = Modifier.align(Alignment.Center),
        )
      }
    }
    Loading -> {
      Box(modifier = Modifier.fillMaxSize()) {
        ProgressBar(modifier = Modifier.align(Alignment.Center))
      }
    }
  }
}
